import os
from django.views.generic import TemplateView
from django.conf import settings
from django.contrib.admin.views.decorators import staff_member_required
from django.core.urlresolvers import reverse

from ydcommon.settings import IGNORE_QUNIT_HTML_FILES


class QunitTestsView(TemplateView):
    template_name = 'ydcommon/js-tests/index.html'

    def get_context_data(self, **kwargs):
        context = super(QunitTestsView, self).get_context_data(**kwargs)
        if not kwargs['path'] or kwargs['path'] == 'index':
            for template_dir in settings.TEMPLATE_DIRS:
                path = os.path.join(template_dir, 'js-tests')
                files = [f.replace('.html', '') for f in os.listdir(path)
                         if os.path.isfile(os.path.join(path, f))]
                for ignore_file in IGNORE_QUNIT_HTML_FILES:
                    if ignore_file in files:
                        files.remove(ignore_file)
                tests = []
                for f in files:
                    tests.append({'file': f,
                                  'url': reverse(qunit_view, args=[f])})
            context['tests'] = tests
        return context

    def get(self, request, *args, **kwargs):
        if not kwargs['path']:
            self.template_name = 'ydcommon/js-tests/index.html'
        else:
            self.template_name = 'js-tests/%s.html' % kwargs['path']
        return super(QunitTestsView, self).get(request, *args, **kwargs)

qunit_view = staff_member_required(QunitTestsView.as_view())



import mock
import sys

from django.test import TestCase
from django.core.management import call_command

BUILTIN_MODULE = '__builtin__'
CMD = 'ydcommon.management.commands.check_test_requirements.commands.getstatusoutput'
CMD_VALUE = (0, '1.9.2')
CMD_VALUE_WRONG = (0, '1.8.2')
if sys.version_info > (3, 0):
    BUILTIN_MODULE = 'builtins'

if sys.version_info > (2, 7):
    CMD = 'ydcommon.management.commands.check_test_requirements.subprocess.check_output'
    CMD_VALUE = ('1.9.2')
    CMD_VALUE_WRONG = ('1.8.2')


class CheckTestRequirementsTests(TestCase):

    @mock.patch(CMD)
    def test_command(self, mock_status):
        mock_status.return_value = CMD_VALUE
        call_command('check_test_requirements')
        self.assertTrue('jshint -v' in mock_status.call_args_list[0][0])
        self.assertTrue('phantomjs -v' in mock_status.call_args_list[1][0])

    @mock.patch(CMD)
    def test_wrong_phantomjs(self, mock_status):
        mock_status.return_value = CMD_VALUE_WRONG
        with self.assertRaises(SystemExit):
            call_command('check_test_requirements')



import os
import base64
import uuid
import django

from datetime import datetime

from django.template.defaultfilters import slugify as _slugify

from fabric.api import local, sudo, run, env, cd, get
from fabric.operations import prompt
from fabric.colors import red
from fabric.contrib.console import confirm


def _get_branch_name(on_local=True):
    cmd = "git branch --no-color 2> /dev/null | sed -e '/^[^*]/d'"
    if on_local:
        name = local(cmd, capture=True).replace("* ", "")
    else:
        name = run(cmd)
    return name.replace("* ", "").strip()


def switch(stage):
    """
        Switch to given stage (dev/qa/production) + pull
    """
    stage = stage.lower()
    local("git pull")
    if stage in ['dev', 'devel', 'develop']:
        branch_name = 'develop'
    elif stage in ['qa', 'release']:
        branches = local('git branch -r', capture=True)
        possible_branches = []
        for b in branches.split("\n"):
            b_parts = b.split('/')
            if b_parts[1] == 'release':
                possible_branches.append(b_parts[2])
        if len(possible_branches) == 0:
            raise Exception('No release branches found. Please create a new release first.')
        possible_branches = sorted(possible_branches, reverse=True)
        branch_name = 'release/%s' % possible_branches[0]
    elif stage in ['production', 'master']:
        branch_name = 'master'
    else:
        raise NotImplemented
    local("git checkout %s" % branch_name)
    local("git pull")


def release_qa(quietly=False):
    """
        Release code to QA server
    """
    name = prompt(red('Sprint name?'), default='Sprint 1').lower().replace(' ', "_")
    release_date = prompt(red('Sprint start date (Y-m-d)?'), default='2013-01-20').replace('-', '')
    release_name = '%s_%s' % (release_date, name)
    local('git flow release start %s' % release_name)
    local('git flow release publish %s' % release_name)
    if not quietly:
        print red('PLEASE DEPLOY CODE: fab deploy:all')


def update_qa(quietly=False):
    """
        Merge code from develop to qa
    """
    switch('dev')
    switch('qa')
    local('git merge --no-edit develop')
    local('git push')
    if not quietly:
        print red('PLEASE DEPLOY CODE: fab deploy:all')


def _check_branch(environment, user, change=False):
    if environment == 'qa':
        local_branch = _get_branch_name()
        remote_branch = _get_branch_name(False)
        if local_branch != remote_branch:
            if not change:
                change = confirm(red('Branch on server is different, do you want to checkout %s ?' % local_branch),
                                 default=True)
            if change:
                sudo('git checkout %s' % local_branch, user=user)


def _sql_paths(*args):
    args = [str(arg) for arg in args]
    args.append(_get_branch_name())
    return _slugify('-'.join(args)) + '.sql.gz'


def backup_db():
    """
        Backup local database
    """
    if not os.path.exists('backups'):
        os.makedirs('backups')
    local('python manage.py dump_database | gzip > backups/' + _sql_paths('local', datetime.now()))


def _get_db():
    """
        Get database from server
    """
    with cd(env.remote_path):
        file_path = '/tmp/' + _sql_paths('remote', base64.urlsafe_b64encode(uuid.uuid4().bytes).replace('=', ''))
        run(env.python + ' manage.py dump_database | gzip > ' + file_path)
    local_file_path = './backups/' + _sql_paths('remote_', datetime.now())
    get(file_path, local_file_path)
    run('rm ' + file_path)
    return local_file_path


def clear_db():
    local('python manage.py clear_database')


def restore_db(path):
    """
        Restore database with given file path (support compressed and not compressed files)
        Usage:
            fab restore_db:~/dump.sql
            fab restore_db:~/dump.sql.gz
    """
    clear_db()
    if path.endswith('.gz'):
        local('gzip -dc %s | python manage.py dbshell' % path)
    else:
        local('python manage.py dbshell < %s ' % path)


def pull_db():
    """
        Replace local database with database from server (server based on branch)
    """
    backup_db()
    local_file_path = _get_db()
    clear_db()
    local('gzip -dc %s | python manage.py dbshell' % local_file_path)


def command(command):
    """
        Run custom Django management command
    """
    with cd(env.remote_path):
        sudo(env.python + ' manage.py %s' % command, user=env.remote_user)


def update_cron():
    """
        Update cron
    """
    sudo('crontab  %sconfig/crontab' % env.remote_path, user=env.remote_user)


def setup_server(clear_old=False, repo="github"):
    """
        Setup server
    """

    if repo == 'github':
        url_keys = 'https://github.com/ArabellaTech/%s/settings/keys' % env.app_dir
        url_clone = 'git@github.com:ArabellaTech/%s.git' % env.app_dir
    elif repo == 'bitbucket':
        url_keys = 'https://bitbucket.org/arabellatech/%s/admin/deploy-keys/' % env.app_dir
        url_clone = 'git@bitbucket.org:arabellatech/%s.git' % env.app_dir
    else:
        raise NotImplementedError('Unknown repo type')

    if clear_old:
        sudo('userdel -r %s' % env.remote_user)

    sudo('useradd --shell /bin/bash --create-home %s' % env.remote_user, user='root')
    sudo('ssh-keygen -t rsa -P "" -f /home/%s/.ssh/id_rsa' % env.remote_user, user=env.remote_user)
    sudo('cp -f /home/%s/.ssh/id_rsa.pub ~/key.tmp' % env.remote_user, user='root')
    key = sudo('cat ~/key.tmp', user='root')
    sudo('rm ~/key.tmp', user='root')
    print red('Please put following deploy key in %s' % url_keys)
    print key
    prompt(red('Press any key to continue'))
    sudo('export WORKON_HOME=/home/%s/Envs &&\
         source /usr/local/bin/virtualenvwrapper_lazy.sh &&\
         mkvirtualenv %s --no-site-packages' % (env.remote_user, env.app_dir),
         warn_only=True, user=env.remote_user)
    sudo('cd /home/%s/ && git clone %s www' % (env.remote_user, url_clone), user=env.remote_user)
    with cd(env.remote_path):
        sudo('git checkout %s' % env.branch, user=env.remote_user)
        sudo('git pull', user=env.remote_user)
        sudo('cd %s && ln -sf ../config/%s/yd_local_settings.py local_settings.py' % (env.app_dir, env.environment),
             user=env.remote_user)
        sudo(env.pip + ' install -r requirements.txt', user=env.remote_user)
        if django.VERSION >= (1, 8):
            sudo(env.python + ' manage.py migrate', user=env.remote_user)
        else:
            sudo(env.python + ' manage.py syncdb --migrate', user=env.remote_user)
        sudo(env.python + ' manage.py collectstatic -v0 --noinput', user=env.remote_user)
        sudo(env.python + ' manage.py compress -f', user=env.remote_user)

        params = (env.remote_user, env.environment, env.remote_user)
        sudo('cd /etc/nginx/sites-enabled && ln -sf /home/%s/www/config/%s/nginx.conf %s.conf' % params, user='root')
        sudo('cd /etc/supervisor/conf.d/ && ln -sf /home/%s/www/config/%s/supervisord.conf %s.conf' % params,
             user='root')
        sudo('/etc/init.d/nginx reload', user='root')
        sudo('supervisorctl reread && supervisorctl update', user='root')

    update_cron()



import os
import django

DATABASE_ENGINE = 'sqlite3'

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'ydcommon',
)
DATABASES = {
    'default': {
        'NAME': ':memory:',
        'ENGINE': 'django.db.backends.sqlite3',
    },
}

project = lambda: os.path.dirname(os.path.realpath(__file__))
location = lambda x: os.path.join(str(project()), str(x))

TEMPLATE_DIRS = (
    location("templates"),
)

STATIC_ROOT = location("static")

STATICFILES_DIRS = [
    location("static"),
]

SECRET_KEY = 'fake'

ROOT_URLCONF = 'test_project.urls'

if django.VERSION >= (1, 8):
    TEST_RUNNER = 'django.test.runner.DiscoverRunner'
else:
    TEST_RUNNER = 'django.test.simple.DjangoTestSuiteRunner'

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)



from django.core.management.base import BaseCommand
from django.db import connection


class Command(BaseCommand):
    help = 'Clear database'

    def handle(self, **options):
        cursor = connection.cursor()
        cursor.execute('show tables;')
        parts = ('DROP TABLE IF EXISTS %s;' % table for (table,) in cursor.fetchall())
        sql = 'SET FOREIGN_KEY_CHECKS = 0;\n' + '\n'.join(parts) + 'SET FOREIGN_KEY_CHECKS = 1;\n'
        connection.cursor().execute(sql)



from __future__ import print_function

import sys

from django.core.management.base import NoArgsCommand

try:
    import subprocess
    import commands
except ImportError:
    pass


class Command(NoArgsCommand):
    help = 'Check test requirements'
    errors = []

    def check_req(self, name, cmd):
        print('Checking ' + name)
        if sys.version_info > (2, 7):
            try:
                result = subprocess.check_output(cmd, stderr=subprocess.STDOUT, shell=True)
                code = 0
            except subprocess.CalledProcessError as e:
                result = e.output
                code = 1
        else:
            code, result = commands.getstatusoutput(cmd)
        if code != 0:
            self.errors.append('jshint')
        return code, result

    def handle_noargs(self, **options):
        self.check_req('jshint', 'jshint -v')
        code, result = self.check_req('phantomjs', 'phantomjs -v')
        if code == 0:
            versions = result.split('.')
            if int(versions[0]) < 1 or int(versions[1]) < 9:
                self.errors.append('phantomjs')

        if self.errors:
            print('Missing: ' + ', '.join(self.errors))
            exit(1)
        else:
            print('OK')



import mock
import sys

from django.test import TestCase
from django.core.management import call_command

BUILTIN_MODULE = '__builtin__'
CMD = 'ydcommon.management.commands.run_qunit.commands.getstatusoutput'
CMD_VALUE = (0, 'X')
if sys.version_info > (3, 0):
    BUILTIN_MODULE = 'builtins'

if sys.version_info > (2, 7):
    import subprocess
    CMD = 'ydcommon.management.commands.run_qunit.subprocess.check_output'
    CMD_VALUE = ('X')


class JSHintTests(TestCase):

    @mock.patch(CMD)
    @mock.patch(BUILTIN_MODULE + ".open")
    def test_command(self, mock_open, mock_status):
        mock_status.return_value = CMD_VALUE
        call_command('jshint')
        self.assertTrue('--show-non-errors' in mock_status.call_args[0][0])

        call_command('jshint', xml_output=True)
        self.assertTrue('--reporter=checkstyle' in mock_status.call_args[0][0])

        if sys.version_info > (2, 7):
            mock_status.side_effect = subprocess.CalledProcessError('x', 'x', 'x')
        call_command('jshint', xml_output=True)



import sys
from django.conf import settings
from django.core.management.base import NoArgsCommand
from optparse import make_option

from ydcommon.settings import JSHINT_FILES_FIND

try:
    import subprocess
    import commands
except ImportError:
    pass


class Command(NoArgsCommand):
    option_list = NoArgsCommand.option_list + (
        make_option('-x', '--xml-output', action='store_true',
                    default=False,
                    dest='xml_output',
                    help='Render as XML'),
        make_option('-d', '--dir', action='store',
                    default=None,
                    dest='search_dir',
                    help='Search files in directory'),
    )
    help = 'Render JSHint'

    def handle_noargs(self, xml_output, search_dir=None, **options):
        files = []
        if search_dir is not None:
            search_dir = [search_dir]
        else:
            search_dir = settings.STATICFILES_DIRS
        for path in search_dir:
            cmd = "find %s %s" % (path, JSHINT_FILES_FIND)
            if sys.version_info > (2, 7):
                try:
                    result = subprocess.check_output(cmd,
                                                     stderr=subprocess.STDOUT,
                                                     shell=True)
                except subprocess.CalledProcessError:
                    return
            else:
                code, result = commands.getstatusoutput(cmd)
            for f in result.split('\n'):
                if f.strip():
                    files.append(f.strip())

            if xml_output:
                cmd = 'jshint --reporter=checkstyle %s' % (' '.join(files))
            else:
                cmd = 'jshint --show-non-errors %s' % (' '.join(files))
            if sys.version_info > (2, 7):
                try:
                    result = subprocess.check_output(cmd,
                                                     stderr=subprocess.STDOUT,
                                                     shell=True)
                    code = 0
                except subprocess.CalledProcessError as e:
                    result = e.output
                    code = e.returncode
            else:
                code, result = commands.getstatusoutput(cmd)
            if result:
                sys.stdout.write(result)
            if not xml_output:
                message = 'OK' if code == 0 else 'Error'
                sys.stdout.write('%s! (Checked files: %d)' % (message, len(files)))



import mock
import sys

from django.test import TestCase
from django.core.management import call_command
from django.contrib.auth.models import User

BUILTIN_MODULE = '__builtin__'
CMD = 'ydcommon.management.commands.run_qunit.commands.getstatusoutput'
CMD_VALUE = (0, """X
<!--
 Tests completed in 289 milliseconds.
16 tests of 16 passed, 0 failed.
-->X""")
if sys.version_info > (3, 0):
    BUILTIN_MODULE = 'builtins'

if sys.version_info > (2, 7):
    CMD = 'ydcommon.management.commands.run_qunit.subprocess.check_output'
    CMD_VALUE = ("""X
<!--
 Tests completed in 289 milliseconds.
16 tests of 16 passed, 0 failed.
-->X""")


class QunitTests(TestCase):

    @mock.patch(CMD)
    @mock.patch(BUILTIN_MODULE + ".open")
    def test_command(self, mock_open, mock_status):
        mock_status.return_value = CMD_VALUE
        call_command('run_qunit')
        self.assertTrue('example.html' in mock_status.call_args[0][0])

    def test_view(self):
        User.objects.create_superuser('foo', 'foo@foo.com' 'me', 'pass')

        self.client.login(username='foo', password='pass')
        response = self.client.get('/js-tests/')
        self.assertContains(response, '/js-tests/example')

        self.client.get('/js-tests/example')
        self.assertEqual(response.status_code, 200)



import os
import sys
import re

from django.conf import settings
from django.core.management.base import NoArgsCommand
from django.template.loader import render_to_string
from django.template import RequestContext
from django.http import HttpRequest
from optparse import make_option
from django.core.management import call_command

from ydcommon.settings import IGNORE_QUNIT_HTML_FILES

try:
    import subprocess
    import commands
except ImportError:
    pass

RE_RESULTS = re.compile("<\!--(.*)-->", re.DOTALL)


class Command(NoArgsCommand):
    option_list = NoArgsCommand.option_list + (
        make_option('-l', '--without-local-paths', action='store_false',
                    default=True,
                    dest='local_paths',
                    help='Render with changing js path to local'),
    )
    help = 'Render mvc tests'

    def handle_noargs(self, **options):
        print('Preparing files')
        if 'django.contrib.staticfiles' in settings.INSTALLED_APPS:
            call_command('collectstatic', interactive=False)
        if 'compressor' in settings.INSTALLED_APPS:
            call_command('compress', force=True, verbosity=0)

        print('Running tests')
        qunit = os.path.join(os.path.dirname(os.path.realpath(__file__)),
                             '..', '..', 'scripts', 'run-qunit.js')

        request = HttpRequest()
        data = RequestContext(request)
        for template_dir in settings.TEMPLATE_DIRS:
            path = os.path.join(template_dir, 'js-tests')
            if not os.path.exists(path):
                continue
            files = [f.replace('.html', '') for f in os.listdir(path)
                     if os.path.isfile(os.path.join(path, f))]
            for ignore_file in IGNORE_QUNIT_HTML_FILES:
                if ignore_file in files:
                    files.remove(ignore_file)
            for filename in files:
                file_path = 'js-tests/%s.html' % (filename)
                output = render_to_string(file_path, data).encode('utf-8')
                if options['local_paths']:
                    output = str(output)
                    output = output.replace('src="/static/', "src=\"{0}/"
                                            .format(settings.STATIC_ROOT))
                    output = output.replace('href="/static/', "href=\"{0}/"
                                            .format(settings.STATIC_ROOT))
                with open('reports/%s.html' % filename, 'w') as f:
                    f.write(output)
                cmd = "phantomjs %s file://`pwd`/reports/%s.html junit-xml" % \
                      (qunit, filename)
                if sys.version_info > (2, 7):
                    try:
                        result = subprocess.check_output(cmd,
                                                         stderr=subprocess.STDOUT,
                                                         shell=True)
                    except subprocess.CalledProcessError as e:
                        result = e.output
                else:
                    code, result = commands.getstatusoutput(cmd)
                with open('reports/junit-%s.xml' % filename, 'w') as f:
                    f.write(result)
                sys.stdout.write(filename.title() + ' - ' + RE_RESULTS.findall(result)[0].replace('\n', ' ').strip())



from django.conf import settings

IGNORE_QUNIT_HTML_FILES = getattr(settings,
                                  'IGNORE_QUNIT_HTML_FILES', ['index', 'base'])

JSHINT_FILES_FIND = getattr(settings,
                            'JSHINT_FILES_FIND',
                            '-name "*.js" | xargs grep -l "/*jslint" | grep -v libs')



import sys
from django.conf import settings
from django.core.management.base import NoArgsCommand
from optparse import make_option


try:
    import subprocess
    import commands
except ImportError:
    pass


class Command(NoArgsCommand):
    option_list = NoArgsCommand.option_list + (
        make_option('-x', '--xml-output', action='store_true',
                    default=False,
                    dest='xml_output',
                    help='Render as XML'),
    )
    help = 'Render JSHint'

    def handle_noargs(self, xml_output, **options):
        cmd = "./manage.py schemamigration %s --auto"
        for app in settings.PROJECT_APPS:
            code, result = False, False
            if sys.version_info > (2, 7):
                try:
                    result = subprocess.check_call(cmd % app,
                                                   stderr=subprocess.STDOUT,
                                                   shell=True)
                except subprocess.CalledProcessError as e:
                    code = e.returncode
                    result = e.output
            else:
                code, result = commands.getstatusoutput(cmd % app)

            if code != 1:
                raise Exception('Missing migration')



import os
import sys
from setuptools import setup, find_packages
from pkg_resources import require, DistributionNotFound


def local_open(fname):
    return open(os.path.join(os.path.dirname(__file__), fname))

readme_file = os.path.abspath(os.path.join(os.path.dirname(__file__), 'README.rst'))

try:
    long_description = open(readme_file).read()
except IOError as err:
    sys.stderr.write("[ERROR] Cannot find file specified as long_description (%s)\n" % readme_file)
    sys.exit(1)

extra_kwargs = {'tests_require': ['mock>1.0']}

ydcommon = __import__('ydcommon')

requirements = local_open('requirements.txt')
required_to_install = []
for dist in requirements.readlines():
    dist = dist.strip()
    try:
        require(dist)
    except DistributionNotFound:
        required_to_install.append(dist)

setup(
    name='ydcommon',
    version=ydcommon.get_version(),
    url='https://github.com/ArabellaTech/ydcommon',
    author='Arabella',
    author_email='team@arabel.la',
    description=ydcommon.__doc__,
    long_description=long_description,
    zip_safe=False,
    install_requires=required_to_install,
    packages=find_packages(),
    license='MIT',
    scripts=[],
    test_suite="test_project.runtests.runtests",
    include_package_data=True,
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
    **extra_kwargs
)



from django.contrib import admin
try:
    from django.conf.urls import patterns, url
except ImportError:
    from django.conf.urls.defaults import patterns, url

admin.autodiscover()


urlpatterns = patterns('',
                       url(r"^js-tests/(?P<path>.*)",
                           'ydcommon.views.qunit_view',
                           name='qunit'),
                       )



#!/usr/bin/env python
import os
import sys

os.environ['DJANGO_SETTINGS_MODULE'] = 'test_project.settings'
test_dir = os.path.dirname(__file__)
sys.path.insert(0, test_dir)


def runtests(*test_args):
    parent = os.path.dirname(os.path.abspath(__file__))
    sys.path.insert(0, parent)

    import django
    if hasattr(django, 'setup'):
        django.setup()

    from django.test.utils import get_runner
    from test_project import settings
    TestRunner = get_runner(settings)

    failures = TestRunner(
        verbosity=1,
        interactive=True,
        failfast=False).run_tests(['ydcommon'], *test_args)
    sys.exit(failures)


if __name__ == '__main__':
    runtests()



"""
YD Technology common libraries
"""

VERSION = (0, 1, 39)

__version__ = '.'.join((str(each) for each in VERSION[:4]))


def get_version():
    """
    Returns shorter version (digit parts only) as string.
    """
    version = '.'.join((str(each) for each in VERSION[:3]))
    if len(VERSION) > 3:
        version += str(VERSION[3])
    return version



import os
import django
import shlex
import sys
from django.core.management.base import BaseCommand, CommandError
from django.db import connections, DEFAULT_DB_ALIAS
from optparse import make_option

DUMP_COMMAND_NAME = 'mysqldump'

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--database', action='store', dest='database',
            default=DEFAULT_DB_ALIAS, help='Nominates a database which to '
                'dump.  Defaults to the "default" database.'),
        )
    help = """\
Dumps the whole database (mysql only!). Looks at the environment
variable MYSQLDUMP_OPTIONS and uses what it finds there as additional
options."""
    args = "[table1 table2 ...]"

    requires_model_validation = False

    def handle(self, *args, **kwargs):
        connection = connections[kwargs.get('database', DEFAULT_DB_ALIAS)]
        settings_dict = connection.settings_dict
        cmd_args = [DUMP_COMMAND_NAME]
        db = settings_dict['OPTIONS'].get('db', settings_dict['NAME'])
        user = settings_dict['OPTIONS'].get('user', settings_dict['USER'])
        passwd = settings_dict['OPTIONS'].get('passwd', settings_dict['PASSWORD'])
        host = settings_dict['OPTIONS'].get('host', settings_dict['HOST'])
        port = settings_dict['OPTIONS'].get('port', settings_dict['PORT'])

        if user:
            cmd_args += ["--user=%s" % user]
        if passwd:
            cmd_args += ["--password=%s" % passwd]
        if host:
            cmd_args += ["--host=%s" % host]
        if port:
            cmd_args += ["--port=%s" % port]

        cmd_args.extend(shlex.split(os.getenv("MYSQLDUMP_OPTIONS", '')))

        if len(args):
            tables = list(args)
        else:
            tables = connection.introspection.get_table_list(connection.cursor())

        if django.VERSION >= (1, 8):
            cmd_args += ["--extended-insert", db] + [str(t.name) for t in tables]
        else:
            cmd_args += ["--extended-insert", db] + tables

        try:
            if os.name == 'nt':
                sys.exit(os.system(" ".join(cmd_args)))
            else:
                os.execvp(DUMP_COMMAND_NAME, cmd_args)
        except OSError:
            # Note that we're assuming OSError means that the client program
            # isn't installed. There's a possibility OSError would be raised
            # for some other reason, in which case this error message would be
            # inaccurate. Still, this message catches the common case.
            raise CommandError('You appear not to have the %r program installed or on your path.' % DUMP_COMMAND_NAME)
